<?php

namespace Drupal\webform_submission_anonymisation\Form;

use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform_submission_anonymisation\Service\Service;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure settings for this site.
 */
class SettingsForm extends ConfigFormBase {
    
  /**
   * @var EntityTypeManager $entity_type_manager
   */
  protected $entity_type_manager;

  /**
   * @var Service $service
   */
  protected $service;

  /**
   * Class constructor.
   */
  public function __construct(EntityTypeManager $entity_type_manager, Service $service) {
    $this->entity_type_manager = $entity_type_manager;
    $this->service = $service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('webform_submission_anonymisation.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'webform_submission_anonymisation_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'webform_submission_anonymisation_settings.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    
    $config = $this->config('webform_submission_anonymisation.settings');

    // Retention duration
    $form['retention_duration'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Retention duration'),
      '#attributes' => [
        'class' => ['container-inline'],
      ],
      'retention_duration_count' => [
        '#type' => 'number',
        '#min' => 0,
        '#max' => 1000,
        '#step' => 1,
        '#size' => 3,
        '#default_value' => $config->get('retention_duration_count') ?? '3'
      ],
      'retention_duration_unit' => [
        '#type' => 'select',
        '#options' => [
          'hours' => $this->t('Hours'),
          'days' => $this->t('Days'),
          'years' => $this->t('Years'),
        ],
        '#default_value' => $config->get('retention_duration_unit') ?? 'years'
      ]
    ];

    // Webform fields to anonymize
    $webform_forms = $this->entity_type_manager->getStorage('webform')->loadMultiple();

    $form['webform_forms'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Webforms configuration'),
      '#description' => $this->t('Enable the fields that should be anonymized after retention period is elapsed.')
    ];

    foreach ($webform_forms as $key => $webform) {
      
      
      $form['webform_forms'][$key] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $this->t('<em>%webform_title</em> form',['%webform_title'=>$webform->label()])
      ];

      $default_value = TRUE; //$config->get('webforms')[$key]['remote_addr'] ?? FALSE;
      $form['webform_forms'][$key]['webform__' . $key . '__remote_addr'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Remote Address (IP)'),
        '#value' => $default_value,
        '#disabled' => true
      ];

      $webform_fields = $webform->getElementsDecodedAndFlattened();
      foreach($webform_fields as $field_name => $field){

        if(!in_array($field_name,['actions'])){
        
          // Prepare title
          $title = trim($field['#title'],'*');

          // TODO: Set default_value to TRUE when field name contains "name", "nom" or "mail" (only once when installed)
          $default_value = FALSE;
          /*$default_value = strpos($field_name,'name') !== false ? TRUE : $default_value;
          $default_value = strpos($field_name,'nom') !== false ? TRUE : $default_value;
          $default_value = strpos($field_name,'mail') !== false ? TRUE : $default_value;
          //*/

          $form['webform_forms'][$key]['webform__' . $key . '__' . $field_name] = [
            '#type' => 'checkbox',
            '#title' => $title,
            '#value' => $config->get('webforms')[$key][$field_name] ?? $default_value
          ];

        }
      }

    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
      
    // Get configuration
    $config_factory = $this->configFactory->getEditable('webform_submission_anonymisation.settings');

    // Record retention duration
    $config_factory->set('retention_duration_count', $form_state->getValue('retention_duration_count'));
    $config_factory->set('retention_duration_unit', $form_state->getValue('retention_duration_unit'));

    // Record "weform__*" fields
    $webforms_settings = [];
    foreach ($form_state->getValues() as $key => $value){
      
      if(strpos($key,'webform__') === 0 && $value){
        $explode = explode('__', $key);
        $webforms_settings[$explode[1]][$explode[2]] = $value;
      }
    }
    $config_factory->set('webforms', $webforms_settings);

    // Save configuration
    $config_factory->save();

    // Anonymize
    $this->service->anonymize();

    parent::submitForm($form, $form_state);
  }
}
