<?php

namespace Drupal\webform_submission_anonymisation\Service;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\Entity\File;

class Service{
  
  use StringTranslationTrait;

  /** 
   * The Webform Submission Anonymisation Config
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;
  
  /** 
   * The Entity type manager
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entity_type_manager;
  
  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected $logger;

  /**
   * Include the messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;


  public function __construct(ConfigFactory $config_factory, EntityTypeManager $entity_type_manager, LoggerChannelFactory $logger, MessengerInterface $messenger){
    $this->config = $config_factory->get('webform_submission_anonymisation.settings');
    $this->entity_type_manager = $entity_type_manager;
    $this->logger = $logger->get('webform_submission_anonymisation');
    $this->messenger = $messenger;
  }

  public function anonymize(){

    // 1 - Calculate retention period
    $count = $this->config->get('retention_duration_count');
    $unit = $this->config->get('retention_duration_unit');

    switch($unit){
      case 'hours':
        $duration = $count * 60 * 60;
        break;
      case 'days':
        $duration = $count * 24 * 60 * 60;
        break;
      case 'years':
        $duration = $count * 365 * 24 *  60 * 60 + (8*60*60);
        break;
      default:
        $duration = 0;
        break;
    }

    $date_limit = time() - $duration;


    // 2 - Get all submissions that are older
    $webfom_submission_storage = $this->entity_type_manager->getStorage('webform_submission');
    $webform_sids = $webfom_submission_storage->getQuery()->condition('remote_addr','','<>')->condition('created',$date_limit,'<')->execute();
    $webform_submissions = $webfom_submission_storage->loadMultiple($webform_sids);

    // 3 - Anonymize submissions
    $settings = $this->config->getRawData();

    foreach ($webform_submissions as $key => $submission) {

      $webform_id = $submission->getWebform()->id();
      $elements = $submission->getWebform()->getElementsDecodedAndFlattened();

      if(isset($settings['webforms'][$webform_id])){
        foreach ($settings['webforms'][$webform_id] as $key => $value) {
          switch($key){
            case 'remote_addr':
              $submission->setRemoteAddr('');
              break;
            default:
              if($elements[$key]['#type'] == 'managed_file'){
                $file = File::load($submission->getElementData($key));
                if($file) $file->delete();
              }
              $submission->setElementData($key,'');
              break;
          }
        }
      }
      $submission->save();

    }    

    // 4 - Logger & Messenger
    $count = count($webform_submissions);
    $message = $count>1 ? $this->t('%count submissions anonymised',['%count' => $count]) : $this->t('%count submission anonymised',['%count' => $count]);
    $this->messenger->addMessage($message);
    $this->logger->notice($message);
    
  }

}
